#include "LConsole.h"


#ifdef LINUX
// Stores keyboard settings
static struct termios new_settings, initial_settings;
static int peek_character = -1;
#endif

void SetTTYRaw(bool HideEcho)
{
#ifdef LINUX
    tcgetattr(0,&initial_settings);
    new_settings = initial_settings;
    new_settings.c_lflag &= ~ICANON;

    if(HideEcho)
        new_settings.c_lflag &= ~ECHO;

    new_settings.c_lflag &= ~ISIG;
    new_settings.c_cc[VMIN] = 1;
    new_settings.c_cc[VTIME] = 0;
    tcsetattr(0, TCSANOW, &new_settings);
#endif
}

void RestoreTTY()
{
#ifdef LINUX
    tcsetattr (STDIN_FILENO, TCSAFLUSH, &initial_settings);
#endif
}




int search_key_codes(unsigned char start_ch)
{
    int PosLoop, FullCodeIndex;

    unsigned char FullCodeStr[CODE_LEN+2] = "\0\0\0\0\0\0";
//printf(" searching ");
    // read in the entire code str
    FullCodeStr[0] = start_ch;
    FullCodeIndex = 1;
//    printf("@%d  ", FullCodeStr[0]);
    while(IsKBHit()) {
        FullCodeStr[FullCodeIndex] = ReadKeyboard();
//        printf("%u ", FullCodeStr[FullCodeIndex]);
        FullCodeIndex++;
        if(FullCodeIndex >= CODE_LEN) {
            printf((char*)FullCodeStr);
            return 0;
        }
    }
    FullCodeStr[FullCodeIndex] = '\0';
    // FullCodeIndex is the length of the str, including the NULL.
//    printf(" *%u ", FullCodeIndex);


    for(PosLoop = 0; PosLoop < POS_CODES; PosLoop++)    {

        if( strcmp((char*)KEY_CODES[PosLoop].long_codes, (char*)FullCodeStr) == 0)
            return KEY_CODES[PosLoop].one_code;

    } // end possible codes loop

    return 0;
}


int IsKBHit()
{
#ifdef LINUX
    unsigned char ch;
    int nread;

    if (peek_character != -1) return 1;
    new_settings.c_cc[VMIN]=0;
    tcsetattr(0, TCSANOW, &new_settings);
    nread = read(0,&ch,1);
    new_settings.c_cc[VMIN]=1;
    tcsetattr(0, TCSANOW, &new_settings);
    if(nread == 1)
    { //printf("!%d!", ch);
        if(ch == ESC)
            peek_character = search_key_codes(ch);
        else
            peek_character = ch;

        return 1;
    }
    return 0;
#endif

#ifdef DOS	
	return kbhit();
#endif	
}

int ReadKeyboard()
{
#ifdef LINUX
    int ch;

    if(peek_character != -1)
    {
        ch = peek_character;
        peek_character = -1;
        return ch;
    }
    read(0,&ch,1);
    return ch;
#endif

#ifdef DOS

    int ch;

    ch = getch();

    if(ch == DOSARROW || ch == DOSZERO)
        return search_key_codes(ch);
    else
        return ch;
#endif
}



// http://www.linux.com/howtos/Bash-Prompt-HOWTO/x329.shtml

// 0,0 is at Top Left.
void SetCursorPos(size_t X, size_t Y)
{
  #ifdef LINUX
    printf("\033[%d;%df", Y+1, X+1); // yes, that command wants Y,X
          // and +1 because this system actually counts from 1
          // unlike everything else in all of computer-dom
  #endif

  #ifdef DOS
    gotoxy(X,Y);  // dos conio function
  #endif

}

void ClearScreen()
{
  #ifdef LINUX
    printf("\033[2J");
  #endif

  #ifdef DOS
    clrscr();
  #endif
}



// Resets it to whatever the system wants it to be.
void ResetColor()
{
  #ifdef LINUX
    printf("\033[0m");
  #endif

  #ifdef DOS
    normvideo();
  #endif

}


void SetColors(CURSOR_STYLE_TYPE Style, CURSOR_COLOR_TYPE Fg, CURSOR_COLOR_TYPE Bg)
{
  #ifdef LINUX
    printf("\033[%d;%d;%dm", Style, CURSOR_COLOR_LINUX_CODES[Fg],
                                    CURSOR_COLOR_LINUX_CODES[Bg]+10);
  #endif

  #ifdef DOS

    if(Style == CT_NORMAL)
        lowvideo();
    else if(Style == CT_BRIGHT)
        highvideo();

    textbackground(Bg);
    textcolor(Fg);
  #endif
}